
button_style_full = {
    "display": "inline-block",
    "height": 40,
    "width": 90,
    "padding": "0 10 10 10",
    "color": "#555",
    "textAlign": "center",
    "fontSize": 11,
    "fontWeight": 600,
    # "lineHeight": 38,
    "letterSpacing": .1,
    "textTransform": "uppercase",
    "textDecoration": "none",
    "whiteSpace": "nowrap",
    "backgroundColor": "transparent",
    "borderRadius": 4,
    "border": "1 solid #bbb",
    "cursor": "pointer",
    "boxSizing": "border-box"
}

modal_info_style= {
    # 'background-color': '#D1FFFD',
    # "color": "#0057D8",
    'background-color': '#FFFFFF',
    "color": "#030303",
}


# content_style_home = {
#     "padding-top": "0px",
#     "padding-left": "0px",
#     "padding-right": "0px",
#     "padding-bottom": "0px",
#     "margin-top": "0px",
#     "margin-left": "0px",
#     "margin-right": "0px",
#     "margin-bottom": "0px",
# }


# font_style = {
#    'background-color': '#c9170a',
#     'color': '#fff',
#     'text-align': 'center',
#     'height': '88px',
#     'font-size': '14px',
#     'font-weight': '400',
#     'line-height': '88px',
#     'letter-spacing': '-.25px',
# }